==> p5-GRNOC-TL1-1.2.2.tbz: lib/perl5/site_perl/5.14.2/GRNOC/TL1/Device/Fujitsu/FLASHWAVE7500.pm <==
#--------------------------------------------------------------------
#----- GRNOC TL1 Fujitsu FLASHWAVE 7500 Device Module
#-----
#----- Copyright(C) 2009 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: svn+ssh://svn.grnoc.iu.edu/grnoc/perl-lib/GRNOC-TL1/tags/1.2.2/lib/GRNOC/TL1/Device/Fujitsu/FLASHWAVE7500.pm $
#----- $Id: FLASHWAVE7500.pm 5774 2009-10-21 20:01:21Z mrmccrac $
#-----
#----- This module handles the functionality for the Fujitsu TL1
#----- FLASHWAVE 7500 device.  It inherts from the standard
#----- GRNOC::TL1::Device module.
#--------------------------------------------------------------------

package GRNOC::TL1::Device::Fujitsu::FLASHWAVE7500;

use strict;
use warnings;

use base "GRNOC::TL1::Device";

sub new {

  my $caller = shift;
  my $class = ref($caller) || $caller;
  
  my (%args) = @_;

  # set default port used for device if one was not given
  $args{'port'} = 23 if ($args{'port'} eq '');

  # set default prompt used if one was not given
  $args{'prompt'} = ';' if ($args{'prompt'} eq '');

  # don't wait for login prompt by default
  $args{'wait_for_login_prompt'} = 0
    if ($args{'wait_for_login_prompt'} eq '');

  # call the GRNOC::TL1::Device structure
  my $self = $class->SUPER::new(%args);

  bless($self, $class);
  
  return $self;
}

sub connect {

  my $self = shift;

  # call the GRNOC::TL1::Device connect() method
  my $success = $self->SUPER::connect();

  # error connecting
  return 0 if (!$success);

  # submit the '3' character after waiting a moment
  # this is the "3:Command Mode" option
  sleep(1);
  
  my $telnet = $self->{'telnet'};
  return $telnet->put('3');
}

sub get_inventory {

  my $self = shift;
  
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-EQPT:$tid" . "::$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @inventory;

  foreach my $response (@responses) {

    my $device;

    if ($response =~ /^"([^:]+):([^:]*):.*:([^"]+)"$/) {

      $device->{'aid'} = $1;
      $device->{'type'} = $2;
      $device->{'state'} = $3;

      ($device->{'serial'}) = $response =~ /SERIALNO=([^,:]*)/;
      ($device->{'unique_serial'}) = $response =~ /USI=([^,:]*)/;
      ($device->{'clei'}) = $response =~ /CLEI=([^,:]*)/;
      ($device->{'vendor_id'}) = $response =~ /VENDID=([^,:]*)/;
      ($device->{'date'}) = $response =~ /DOM=([^,:]*)/;

      push(@inventory, $device);
    }
    
    else {

      print STDERR "get_inventory: Error parsing response: $response\n";
    }
  }
  
  return @inventory;
}

sub get_alarms {

  my $self = shift;
  
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-ALM-ALL:$tid" . "::$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @alarms;

  foreach my $response (@responses) {

    my $alarm;

    if ($response =~ /^"([^,]+),([^:]*):([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),.*"$/) {

      $alarm->{'aid'} = $1;
      $alarm->{'aid_type'} = $2;
      $alarm->{'not_code'} = $3;
      $alarm->{'cond_type'} = $4;
      $alarm->{'serv_affect'} = $5;
      $alarm->{'date'} = $6;
      $alarm->{'time'} = $7;

      ($alarm->{'description'}) = $response =~ /\\"(.*)\\"/;

       my @date = split(/-/, $alarm->{'date'});

      if (@date == 3) {
	
	($alarm->{'year'}, $alarm->{'month'}, $alarm->{'day'}) = @date;
      }

      else {
	
	($alarm->{'month'}, $alarm->{'day'}) = @date;
      }
      
      $alarm->{'time'} =~ s/-/:/g;

      push(@alarms, $alarm);
    }

    else {

      print STDERR "get_alarms: Error parsing alarm string: $response\n";
    }
  }

  return @alarms;
}

sub get_conditions {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-COND-ALL:$tid" . "::$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @conditions;

  foreach my $response (@responses) {

    my $condition;

    # "AID,AIDTYPE:NTFCNCODE,CONDTYPE,SRVEFF,OCRDAT,OCRTM,LOCN,DIRN,TMPER,CONDDESCR,AIDDET,OBSDBHVR,EXPTDBHVR"
    if ($response =~ /^"([^,]+),([^:]*):([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),.*"$/) {

      $condition->{'aid'} = $1;
      $condition->{'aid_type'} = $2;
      $condition->{'not_code'} = $3;
      $condition->{'cond_type'} = $4;
      $condition->{'serv_affect'} = $5;
      $condition->{'date'} = $6;
      $condition->{'time'} = $7;

      ($condition->{'description'}) = $response =~ /\\"(.*)\\"/;

       my @date = split(/-/, $condition->{'date'});

      if (@date == 3) {
	
	($condition->{'year'}, $condition->{'month'}, $condition->{'day'}) = @date;
      }

      else {
	
	($condition->{'month'}, $condition->{'day'}) = @date;
      }
      
      $condition->{'time'} =~ s/-/:/g;

      push(@conditions, $condition);
    }

    else {

      print STDERR "get_conditions: Error parsing condition string: $response\n";
    }
  }

  return @conditions;
}

sub get_network_element_type {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-NETYPE:$tid" . "::$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);

  foreach my $response (@responses) {

    if ($response =~ /^"(.*)"$/) {

      my ($vendor, $model, $ne_type, $software) = split(/,/, $1);

      return {vendor => $vendor,
	      model => $model,
	      type => $ne_type,
	      software => $software};
    }
  }

  return undef;
}

sub get_optical_channels {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  # we need to make 4 calls: RTRV-OCH107, RTRV-OCH111, RTRV-OCH430, RTRV-OCH445
  my $cmds = ["RTRV-OCH107:$tid" . "::$ctag;",
	      "RTRV-OCH111:$tid" . "::$ctag;",
	      "RTRV-OCH430:$tid" . "::$ctag;",
	      "RTRV-OCH445:$tid" . "::$ctag;"];
  
  my @channels;

  foreach my $cmd (@$cmds) {

    my @output_messages = $self->parse($self->cmd($cmd));
    my @responses = $self->get_responses(@output_messages);

    foreach my $response (@responses) {

      if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {
	
	my $aid = $1;
	my $state = $2;
	
	my $chan;
	
	$chan->{'aid'} = $aid;
	$chan->{'state'} = $state;
	
	push(@channels, $chan);
      }
      
      else {
	
	print STDERR "get_optical_channels: Error parsing response string: $response\n";
      }
    }
  }
 
  return @channels;
}

sub get_optical_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-WDM:$tid" . "::$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

      my $aid = $1;
      my $state = $2;

      my $intf;

      $intf->{'aid'} = $aid;
      $intf->{'state'} = $state;

      push(@interfaces, $intf);
    }

    else {

      print STDERR "get_optical_interfaces: Error parsing response string: $response\n";
    }
  }

  return @interfaces;
}

# actual logic for all of the get_oc*_interfaces methods
sub _get_ocn_interfaces {

  my ($self, $ocn) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-" . $ocn . ":$tid" . "::$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

      my $aid = $1;
      my $state = $2;

      my $intf;

      $intf->{'aid'} = $aid;
      $intf->{'state'} = $state;

      push(@interfaces, $intf);
    }

    else {

      print STDERR "_get_ocn_interfaces: Error parsing response string: $response\n";
    }
  }

  return @interfaces;
}

sub get_oc3_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC3");
}

sub get_oc12_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC12");
}

sub get_oc48_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC48");
}

sub get_oc192_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC192");
}

sub get_oc768_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC768");
}

sub get_ethernet_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-1GE:$tid" . "::$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

      my $aid = $1;
      my $state = $2;

      my $intf;

      $intf->{'aid'} = $aid;
      $intf->{'state'} = $state;

      push(@interfaces, $intf);
    }

    else {

      print STDERR "get_ethernet_interfaces: Error parsing response string: $response\n";
    }
  }

  return @interfaces;
}

sub get_10gig_ethernet_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-10GE:$tid" . "::$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

      my $aid = $1;
      my $state = $2;

      my $intf;

      $intf->{'aid'} = $aid;
      $intf->{'state'} = $state;

      push(@interfaces, $intf);
    }

    else {

      print STDERR "get_10gig_ethernet_interfaces: Error parsing response string: $response\n";
    }
  }

  return @interfaces;
}

sub get_autonomous_output {

  my ($self, $type, $start, $end) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-AO:$tid:";

  # start / end is optional
  if ($start && $end) {
    
    $cmd .= "$start&&$end";
  }

  elsif ($start) {

    $cmd .= "$start&&";
  }
  
  $cmd .= ":$ctag" . ":::MSGTYPE=" . $type;

  $cmd .= ";";

  # send the command
  $self->send($cmd);

  # recv the raw output from this command
  my $output = $self->recv($cmd, $ctag);

  # grab all the autonomous messages from the output
  my @messages = $self->parse_autonomous_messages($output);

  my @results;

  # go through each auto message
  foreach my $message (@messages) {

    # make sure its of the right message type
    next if ($message->{'verb'} !~ /^REPT $type/);
      
    # go through each response in this message
    my $responses = $message->{'responses'};
      
    foreach my $response (@$responses) {
      
      # "[AID]:NTFCNCDE,CONDTYPE,SRVEFF,,,[LOCN],[DIRN][:[CONDDESCR][,AIDDET][,OBSDBHVR][,EXPTDBHVR]]]"
      if ($response =~ /^"([^:]*):([^,]*),([^,]*),([^,]*),,,([^,]*),(TRMT|RCV|NA)(.*)$/) {
	  
	my $msg;

	$msg->{'aid'} = $1;
	$msg->{'not_code'} = $2;
	$msg->{'cond_type'} = $3;
	$msg->{'serv_affect'} = $4;
	$msg->{'location'} = $5;
	$msg->{'direction'} = $6;
	
	my $rest = $7;
	
	($msg->{'description'}) = $rest =~ /:\\"([^\\"]+)/;

	# shove in date & time from response header
	$msg->{'date'} = $message->{'date'};
	$msg->{'time'} = $message->{'time'};

	my @date = split(/-/, $msg->{'date'});
	
	if (@date == 3) {
	  
	  ($msg->{'year'}, $msg->{'month'}, $msg->{'day'}) = @date;
	}
	
	else {
	  
	  ($msg->{'month'}, $msg->{'day'}) = @date;
	}

	push(@results, $msg);
      }

      else {
	
	print STDERR "get_autonomous_output: Error parsing response: $response\n";
      }
    }
  }
  
  return @results;
}

1;

==> p5-GRNOC-TL1-1.3.10.tbz: lib/perl5/site_perl/5.14/GRNOC/TL1/Device/Fujitsu/FLASHWAVE7500.pm <==
#--------------------------------------------------------------------
#----- GRNOC TL1 Fujitsu FLASHWAVE 7500 Device Module
#-----
#----- Copyright(C) 2009 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: svn+ssh://svn.grnoc.iu.edu/grnoc/perl-lib/GRNOC-TL1/tags/1.3.10/lib/GRNOC/TL1/Device/Fujitsu/FLASHWAVE7500.pm $
#----- $Id: FLASHWAVE7500.pm 11586 2011-03-22 20:20:18Z daldoyle $
#-----
#----- This module handles the functionality for the Fujitsu TL1
#----- FLASHWAVE 7500 device.  It inherts from the standard
#----- GRNOC::TL1::Device module.
#--------------------------------------------------------------------

package GRNOC::TL1::Device::Fujitsu::FLASHWAVE7500;

use strict;
use warnings;

use base "GRNOC::TL1::Device";

sub new {

  my $caller = shift;
  my $class = ref($caller) || $caller;
  
  my (%args) = @_;

  # set default port used for device if one was not given
  $args{'port'} = 23 if ($args{'port'} eq '');

  # set default prompt used if one was not given
  $args{'prompt'} = ';' if ($args{'prompt'} eq '');

  # don't wait for login prompt by default
  $args{'wait_for_login_prompt'} = 0
    if ($args{'wait_for_login_prompt'} eq '');

  # call the GRNOC::TL1::Device structure
  my $self = $class->SUPER::new(%args);

  bless($self, $class);
  
  return $self;
}

sub connect {

  my $self = shift;

  # call the GRNOC::TL1::Device connect() method
  my $success = $self->SUPER::connect();

  # error connecting
  return 0 if (!$success);

  # submit the '3' character after waiting a moment
  # this is the "3:Command Mode" option
  sleep(1);
  
  my $telnet = $self->{'telnet'};
  return $telnet->put('3');
}

sub get_inventory {

  my $self = shift;
  
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-EQPT:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @inventory;

  foreach my $response (@responses) {

    my $device;

    if ($response =~ /^"([^:]+):([^:]*):.*:([^"]+)"$/) { #"

      $device->{'aid'} = $1;
      $device->{'type'} = $2;
      $device->{'state'} = $3;

      ($device->{'serial'}) = $response =~ /SERIALNO=([^,:]*)/;
      ($device->{'unique_serial'}) = $response =~ /USI=([^,:]*)/;
      ($device->{'clei'}) = $response =~ /CLEI=([^,:]*)/;
      ($device->{'vendor_id'}) = $response =~ /VENDID=([^,:]*)/;
      ($device->{'date'}) = $response =~ /DOM=([^,:]*)/;

      push(@inventory, $device);
    }
    
    else {

      print STDERR "get_inventory: Error parsing response: $response\n";
    }
  }
  
  return @inventory;
}

sub get_alarms {

  my $self = shift;
  
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-ALM-ALL:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @alarms;

  foreach my $response (@responses) {

    my $alarm;

    if ($response =~ /^"([^,]+),([^:]*):([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),.*"$/) {

      $alarm->{'aid'} = $1;
      $alarm->{'aid_type'} = $2;
      $alarm->{'not_code'} = $3;
      $alarm->{'cond_type'} = $4;
      $alarm->{'serv_affect'} = $5;
      $alarm->{'date'} = $6;
      $alarm->{'time'} = $7;

      ($alarm->{'description'}) = $response =~ /\\"(.*)\\"/;

       my @date = split(/-/, $alarm->{'date'});

      if (@date == 3) {
	
	($alarm->{'year'}, $alarm->{'month'}, $alarm->{'day'}) = @date;
      }

      else {
	
	($alarm->{'month'}, $alarm->{'day'}) = @date;
      }
      
      $alarm->{'time'} =~ s/-/:/g;

      push(@alarms, $alarm);
    }

    else {

      print STDERR "get_alarms: Error parsing alarm string: $response\n";
    }
  }

  return @alarms;
}

sub get_conditions {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-COND-ALL:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @conditions;

  foreach my $response (@responses) {

    my $condition;

    # "AID,AIDTYPE:NTFCNCODE,CONDTYPE,SRVEFF,OCRDAT,OCRTM,LOCN,DIRN,TMPER,CONDDESCR,AIDDET,OBSDBHVR,EXPTDBHVR"
    if ($response =~ /^"([^,]+),([^:]*):([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),.*"$/) {

      $condition->{'aid'} = $1;
      $condition->{'aid_type'} = $2;
      $condition->{'not_code'} = $3;
      $condition->{'cond_type'} = $4;
      $condition->{'serv_affect'} = $5;
      $condition->{'date'} = $6;
      $condition->{'time'} = $7;

      ($condition->{'description'}) = $response =~ /\\"(.*)\\"/;

       my @date = split(/-/, $condition->{'date'});

      if (@date == 3) {
	
	($condition->{'year'}, $condition->{'month'}, $condition->{'day'}) = @date;
      }

      else {
	
	($condition->{'month'}, $condition->{'day'}) = @date;
      }
      
      $condition->{'time'} =~ s/-/:/g;

      push(@conditions, $condition);
    }

    else {

      print STDERR "get_conditions: Error parsing condition string: $response\n";
    }
  }

  return @conditions;
}

sub get_network_element_type {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-NETYPE:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  foreach my $response (@responses) {

    if ($response =~ /^"(.*)"$/) {

      my ($vendor, $model, $ne_type, $software) = split(/,/, $1);

      return {vendor => $vendor,
	      model => $model,
	      type => $ne_type,
	      software => $software};
    }
  }

  return undef;
}

sub get_optical_channels {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  # we need to make 4 calls: RTRV-OCH107, RTRV-OCH111, RTRV-OCH430, RTRV-OCH445
  my $cmds = ["RTRV-OCH107:$tid" . "::$ctag;",
	      "RTRV-OCH111:$tid" . "::$ctag;",
	      "RTRV-OCH430:$tid" . "::$ctag;",
	      "RTRV-OCH445:$tid" . "::$ctag;"];
  
  my @channels;

  foreach my $cmd (@$cmds) {

    my $cmd_output = $self->cmd($cmd);
    return undef if (!defined $cmd_output);

    my @output_messages = $self->parse($cmd_output);
    my @responses = $self->get_responses(@output_messages);

    foreach my $response (@responses) {

      if ($response =~ /^"([^:]+):.*:([^"]+)"$/) { #"
    
	my $aid = $1;
	my $state = $2;
    
	my $chan;
    
        $chan->{'aid'} = $aid;
        $chan->{'state'} = $state;
    
        push(@channels, $chan);
      }
      
      else {
	
	print STDERR "get_optical_channels: Error parsing response string: $response\n";
      }
    }
  }
 
  return @channels;
}

sub get_optical_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-WDM:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

      my $aid = $1;
      my $state = $2;

      my $intf;

      $intf->{'aid'} = $aid;
      $intf->{'state'} = $state;

      push(@interfaces, $intf);
    }

    else {

      print STDERR "get_optical_interfaces: Error parsing response string: $response\n";
    }
  }

  return @interfaces;
}

# actual logic for all of the get_oc*_interfaces methods
sub _get_ocn_interfaces {

  my ($self, $ocn) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-" . $ocn . ":$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

      my $aid = $1;
      my $state = $2;

      my $intf;

      $intf->{'aid'} = $aid;
      $intf->{'state'} = $state;

      push(@interfaces, $intf);
    }

    else {

      print STDERR "_get_ocn_interfaces: Error parsing response string: $response\n";
    }
  }

  return @interfaces;
}

sub get_oc3_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC3");
}

sub get_oc12_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC12");
}

sub get_oc48_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC48");
}

sub get_oc192_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC192");
}

sub get_oc768_interfaces {

  my $self = shift;

  return $self->_get_ocn_interfaces("OC768");
}

sub get_ethernet_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-1GE:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

      my $aid = $1;
      my $state = $2;

      my $intf;

      $intf->{'aid'} = $aid;
      $intf->{'state'} = $state;

      push(@interfaces, $intf);
    }

    else {

      print STDERR "get_ethernet_interfaces: Error parsing response string: $response\n";
    }
  }

  return @interfaces;
}

sub get_10gig_ethernet_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-10GE:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

      my $aid = $1;
      my $state = $2;

      my $intf;

      $intf->{'aid'} = $aid;
      $intf->{'state'} = $state;

      push(@interfaces, $intf);
    }

    else {

      print STDERR "get_10gig_ethernet_interfaces: Error parsing response string: $response\n";
    }
  }

  return @interfaces;
}

sub get_autonomous_output {

  my ($self, $type, $start, $end) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-AO:$tid:";

  # start / end is optional
  if ($start && $end) {
    
    $cmd .= "$start&&$end";
  }

  elsif ($start) {

    $cmd .= "$start&&";
  }
  
  $cmd .= ":$ctag" . ":::MSGTYPE=" . $type;

  $cmd .= ";";

  # send the command
  $self->send($cmd);

  # recv the raw output from this command
  my $output = $self->recv($cmd, $ctag);

  # grab all the autonomous messages from the output
  my @messages = $self->parse_autonomous_messages($output);

  my @results;

  # go through each auto message
  foreach my $message (@messages) {

    # make sure its of the right message type
    next if ($message->{'verb'} !~ /^REPT $type/);
      
    # go through each response in this message
    my $responses = $message->{'responses'};
      
    foreach my $response (@$responses) {
      
      # "[AID]:NTFCNCDE,CONDTYPE,SRVEFF,,,[LOCN],[DIRN][:[CONDDESCR][,AIDDET][,OBSDBHVR][,EXPTDBHVR]]]"
      if ($response =~ /^"([^:]*):([^,]*),([^,]*),([^,]*),,,([^,]*),(TRMT|RCV|NA)(.*)$/) {
	  
	my $msg;

	$msg->{'aid'} = $1;
	$msg->{'not_code'} = $2;
	$msg->{'cond_type'} = $3;
	$msg->{'serv_affect'} = $4;
	$msg->{'location'} = $5;
	$msg->{'direction'} = $6;
	
	my $rest = $7;
	
	($msg->{'description'}) = $rest =~ /:\\"([^\\"]+)/;

	# shove in date & time from response header
	$msg->{'date'} = $message->{'date'};
	$msg->{'time'} = $message->{'time'};

	my @date = split(/-/, $msg->{'date'});
	
	if (@date == 3) {
	  
	  ($msg->{'year'}, $msg->{'month'}, $msg->{'day'}) = @date;
	}
	
	else {
	  
	  ($msg->{'month'}, $msg->{'day'}) = @date;
	}

	push(@results, $msg);
      }

      else {
	
	print STDERR "get_autonomous_output: Error parsing response: $response\n";
      }
    }
  }
  
  return @results;
}

1;
